<?php
/*
Plugin Name: Xoogu Dynamic Shortcodes
Plugin URI: http://www.xoogu.com/dynamic-shortcodes-plugin-for-wp-super-cache/
Description: Enables you to select shortcodes that should be processed dynamically on every page view
Version: 2016-06-02
Author: xoogu.com
Author URI: http://www.xoogu.com/
*/

$Xoogu_DynamicShortcodesWPSC = new Xoogu_DynamicShortcodesWPSC();

class Xoogu_DynamicShortcodesWPSC
{
	//@var array Stores shortcodes and their handlers for use when first processing the page just after caching it
	protected $aShortcodeTagsOld = array();
	
	/**
	 *Add the needed hooks to work with WP Super Cache
	 */
	public function __construct()
	{
		global $wpsc_xds_shortcodes;
		
		//don't do anything for an AJAX request as they aren't cached
		if (defined('DOING_AJAX') && DOING_AJAX) {
			return;
		}

		add_cacheaction( 'cache_admin_page', array($this, 'admin_page') );
		if (!empty($GLOBALS['wpsc_xds_secret'])) {
			//Make the shortcodes dynamic for every page load
			//check late init and dynamic caching have been enabled
			if (!empty($GLOBALS['wp_super_cache_late_init']) && !empty($GLOBALS['wp_cache_mfunc_enabled']) && !empty($wpsc_xds_shortcodes)) {
				add_cacheaction('wpsc_cachedata_safety', array($this, 'safety'));
				add_cacheaction('add_cacheaction', array($this, 'initPreCache'));
				add_cacheaction('wpsc_cachedata', array($this, 'processShortcodes'));
			} elseif(!empty($GLOBALS['wp_super_cache_debug'])) {
				$sErrMsg = 'Xoogu Dynamic Shortcodes Plugin: $wpsc_xds_secret is set but';
				if (empty($GLOBALS['wp_super_cache_late_init'])) {
					$sErrMsg .= ' Late init does not appear to be enabled';
				}
				if (empty($GLOBALS['wp_cache_mfunc_enabled'])) {
					$sErrMsg .= ' Dynamic caching does not appear to be enabled';
				}
				if (empty($wpsc_xds_shortcodes)) {
					$sErrMsg .= ' No shortcodes have been added to be dynamically processed';
				}
				wp_cache_debug($sErrMsg);
			}
		}
	}
	
	
	/**
	 *Displays the options form in the plugins page of the WP Super Cache admin
	 */
	public function admin_page()
	{
		include(dirname(__FILE__).'/xoogu-dynamic-shortcodes/admin.php');
	}
	
	
	/**
	 *WP Super Cache requires this to enable the dynamic functionality
	 *@return int Numeral 1 to indicate that dynamic functionality should be enabled
	 */
	public function safety($safety)
	{
		return 1;
	}
	
	
	/**
	 *Process the shortcodes on the cached page
	 *@param string $cachedata The cached content we will modify
	 *@return string The modified content with the shortcodes parsed
	 */
	public function processShortcodes($sCachedata)
	{
		global $shortcode_tags, $wpsc_xds_shortcodes;
		
		//backup the list of shortcode tags
		$aBk_shortcode_tags = $shortcode_tags;
		
		//If we're processing a just generated page, then the original shortcode handlers have been removed
		//So we need to re-assign back the original handlers that we stored during initPreCache()
		if (!empty($this->aShortcodeTagsOld)) {
			$aBk_shortcode_tags = array_merge($aBk_shortcode_tags,$this->aShortcodeTagsOld);
		}
		//file_put_contents()
		//overwrite the shortcode tags with only our ones
		$shortcode_tags = array();
		foreach ($wpsc_xds_shortcodes as $shortcode) {
			$shortcode_tags[$shortcode.'_'.$GLOBALS['wpsc_xds_secret']] = $aBk_shortcode_tags[$shortcode];
		}
		//process the shortcodes
		$sCachedata = do_shortcode($sCachedata);
		//restore the list of shortcode tags to its original state
		$shortcode_tags = $aBk_shortcode_tags;
		return $sCachedata;
	}
	
	
	/**
	 *Add the shortcodes to be processed on the page before it is cached
	 *We simply add shortcode handlers that change the shortcode tags to have the secret key
	 *as part of the shortcode name
	 */
	function initPreCache()
	{
		global $wpsc_xds_shortcodes;
		
		foreach ($wpsc_xds_shortcodes as $shortcode) {
			//Check the shortcode tag exists (may not if a wp-admin ajax request)
			if (!empty($GLOBALS['shortcode_tags'][$shortcode])) {
				//backup shortcode so that we can still access the handler when processing the initial generation of the page
				$this->aShortcodeTagsOld[$shortcode] = $GLOBALS['shortcode_tags'][$shortcode];
				//remove existing handler
				remove_shortcode($shortcode);
				//add our dynamic handler
				add_shortcode($shortcode, array($this, 'preCacheShortcodeHandler'));
			}
		}
	}
	
	
	/**
	 *Shortcode handler run when generating the page cache
	 *Rebuilds the shortcode but with the secret key
	 *@param array $attrs The attributes from the shortcode
	 *@param string $content The text content the shortcode wraps
	 *@param string $type The name of shortcode we are generating the dynamic fragment for
	 *@return string The shortcode with the secret key as part of the tag
	 */
	public function preCacheShortcodeHandler($attrs, $content='', $type)
	{
		$str = '['.$type.'_'.$GLOBALS['wpsc_xds_secret'];
		if (!empty($attrs)) {
			foreach ($attrs as $key => $val) {
				if (is_int($key)) {
					//attributes with no value are keyed numerically, so need to recreate as attribute only
					$str .= " $val";
				} else {
					$str .= ' '.$key.'="'.$val.'"';
				}
			}
		}
		$str .= ']'.$content.'[/'.$type.'_'.$GLOBALS['wpsc_xds_secret'].']';
		return $str;
	}
	
}